@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2002, 2003 Philip Blundell <philb@gnu.org>
+ * Copyright (C) 2002, 2003, 2004 Philip Blundell <philb@gnu.org>
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -102,6 +102,13 @@ todo_db_category_find_by_id (int i)
   return NULL;
 }
 
+void
+todo_db_destroy_category (struct todo_category *c)
+{
+  g_free ((gpointer)c->title);
+  g_free (c);
+}
+
 void
 todo_db_del_category (struct todo_category *c)
 {
@@ -116,6 +123,7 @@ todo_db_del_category (struct todo_category *c)
     }
 
   todo_db_categories = g_slist_remove (todo_db_categories, c);
+  todo_db_destroy_category (c);
 }
 
 /* ------ */
@@ -127,19 +135,21 @@ item_data_callback (void *arg, int argc, char **argv, char **names)
 
   if (argc == 2 && argv[0] && argv[1])
     {
-      if (!strcmp (argv[0], "SUMMARY"))
+      if (!strcasecmp (argv[0], "SUMMARY"))
 	i->summary = g_strdup (argv[1]);
-      else if (!strcmp (argv[0], "DESCRIPTION"))
+      else if (!strcasecmp (argv[0], "DESCRIPTION"))
 	i->what = g_strdup (argv[1]);
-      else if (!strcmp (argv[0], "STATE"))
+      else if (!strcasecmp (argv[0], "STATE"))
 	i->state = atoi (argv[1]);
-      else if (!strcmp (argv[0], "CATEGORY"))
+      else if (!strcasecmp (argv[0], "PRIORITY"))
+	i->state = atoi (argv[2]);
+      else if (!strcasecmp (argv[0], "CATEGORY"))
 	{
 	  struct todo_category *c = todo_db_category_find_by_id (atoi (argv[1]));
 	  if (c)
 	    i->categories = g_slist_append (i->categories, c);;
 	}
-      else if (!strcmp (argv[0], "DUE"))
+      else if (!strcasecmp (argv[0], "DUE"))
 	{
 	  struct tm tm;
 	  memset (&tm, 0, sizeof (tm));
@@ -160,6 +170,7 @@ item_callback (void *arg, int argc, char **argv, char **names)
       int id = atoi (argv[0]);
       struct todo_item *i = g_malloc0 (sizeof (struct todo_item));
       i->id = id;
+      i->priority = PRIORITY_STANDARD;
       if (sqlite_exec_printf (sqliteh, "select tag,value from todo where uid=%d",
 			      item_data_callback, i, &err, id))
 	{
@@ -287,6 +298,18 @@ todo_db_start (void)
 void
 todo_db_stop (void)
 {
+  GSList *iter;
+
+  for (iter = todo_db_items; iter; iter = g_slist_next (iter))
+    todo_db_destroy_item (iter->data);
+  g_slist_free (todo_db_items);
+  todo_db_items = NULL;
+
+  for (iter = todo_db_categories; iter; iter = g_slist_next (iter))
+    todo_db_destroy_category (iter->data);
+  g_slist_free (todo_db_categories);
+  todo_db_categories = NULL;
+	
   sqlite_close (sqliteh);
 }
 
@@ -307,7 +330,7 @@ list_sort_func (gconstpointer a, gconstpointer b)
 }
 
 #define insert_values(db, id, key, format, value)	\
-	sqlite_exec_printf (db, "insert into todo values (%d, '%q', '" ## format ## "')", \
+	sqlite_exec_printf (db, "insert into todo values (%d, '%q', '" format "')", \
 			    NULL, NULL, &err, id, key, value)
 
 gboolean
@@ -316,6 +339,9 @@ todo_db_push_item (struct todo_item *i)
   char *err;
   gboolean rollback = FALSE;
   GSList *iter;
+  time_t modified;
+
+  modified = time (NULL);
 
   if (sqlite_exec (sqliteh, "begin transaction", NULL, NULL, &err))
     goto error;
@@ -345,6 +371,12 @@ todo_db_push_item (struct todo_item *i)
 	goto error;
     }
 
+  if (insert_values (sqliteh, i->id, "PRIORITY", "%d", i->priority))
+    goto error;
+
+  if (insert_values (sqliteh, i->id, "MODIFIED", "%d", (int)modified))
+    goto error;
+
   if (sqlite_exec (sqliteh, "commit transaction", NULL, NULL, &err))
     goto error;
 
@@ -390,6 +422,15 @@ todo_db_new_item (void)
   return i;
 }
 
+void
+todo_db_destroy_item (struct todo_item *i)
+{
+  g_free ((gpointer)i->what);
+  g_free ((gpointer)i->summary);
+  g_slist_free (i->categories);
+  g_free (i);
+}
+
 void 
 todo_db_delete_item (struct todo_item *i)
 {
@@ -402,7 +443,5 @@ todo_db_delete_item (struct todo_item *i)
 
   todo_db_items = g_slist_remove (todo_db_items, i);
 
-  g_free ((gpointer)i->what);
-  g_free ((gpointer)i->summary);
-  g_free (i);
+  todo_db_destroy_item (i);
 }
